#!/usr/bin/env python

import micropython.cmd
import micropython.report
from micropython.graph import get_graph
import rsvp
import sys
import os

libdirs = [
    os.path.join(os.path.split(__file__)[0], "lib"),
    "/usr/share/micropython/lib"
    ]

def show_code(code):
    for i, x in enumerate(code):
        print i, x

def show_table_usage(raw_table, slice_size=100):
    for x in xrange(0, len(raw_table), slice_size):
        table_slice = raw_table[x:x+slice_size]
        print "%6d" % (len(table_slice) - table_slice.count(None)), \
            "".join(entry and "#" or "_" for entry in table_slice)

def show_warnings(attribute_usage_failures):
    failures = list(attribute_usage_failures)
    failures.sort()
    for unit_name, name, attrnames, all_attributes in failures:
        attrnames = list(attrnames)
        attrnames.sort()
        print >>sys.stderr, "%s: Name %r with %s attributes %r" % (
            unit_name, name, all_attributes and "all" or "any", ", ".join(attrnames))

def attrs(obj):
    for name, attr in obj.items():
        print name, attr

def reset():
    global rm
    rm = rsvp.machine(p)

# Main program.

if __name__ == "__main__":
    args = sys.argv[2:]
    path = libdirs + sys.path[:]

    if "--help" in sys.argv:
        print "Optimisations:"
        print micropython.cmd.show_optimisations()
        sys.exit(1)

    if len(sys.argv) > 1 and sys.argv[1] != "-":
        filename = os.path.abspath(sys.argv[1])
        path.append(os.path.split(filename)[0])
    else:
        filename = None

    # Load the program.

    try:
        p = micropython.cmd.get_program(path, args)
        i = p.get_importer()

        if filename is None or filename == "-":
            print "Loading module micropython ..."
            m = i.load("micropython")
        else:
            print "Loading from", filename, "..."
            m = i.load_from_file(filename)

        p.finalise()

        # Show warnings.

        if "-w" in sys.argv:
            print >>sys.stderr
            print >>sys.stderr, "Warnings:"
            show_warnings(i.attribute_usage_failures)
            print >>sys.stderr

        # Make the builtins module available through a variable.

        b = i.get_module("__builtins__")

        # Make a report.

        if "-d" in args:
            try:
                directory = args[args.index("-d") + 1]
            except IndexError:
                print "No directory specified. Not generating report."
            else:
                print "Generating report in", directory
                micropython.report.report(p, directory)

        if "-G" in args:
            try:
                graph = args[args.index("-G") + 1]
            except IndexError:
                print "No file specified. Not generating graph."
            else:
                print "Generating graph as", graph
                f = open(graph, "w")
                try:
                    get_graph(p, f, filename)
                finally:
                    f.close()

        # Build the program.

        if "-m" in args or "-t" in args:
            rm = rsvp.machine(p, debug=("-g" in args), abort_upon_exception=("-x" in args))

            if "-t" in args:
                success = rm.test(m)
                print "Test successful?", success

            print "RSVP machine:    rm = %r" % rm

        ot = p.get_object_table()
        pt = p.get_parameter_table()

        print "Object table:    ot = %r" % ot
        print "Parameter table: pt = %r" % pt
        print "Importer:        i = %r" % i
        print "Program:         p = %r" % p
        print "Module:          m = %r" % m
        print "Built-ins:       b = %r" % b

    # Report any errors.

    except micropython.ProcessingError, exc:
        print repr(exc)
        if "-tb" in args:
            raise
        elif "-exit" in args:
            sys.exit(1)

    except KeyboardInterrupt:
        if "-exit" in args:
            sys.exit(2)
        else:
            raise

    else:
        if "-exit" in args and "-t" in args:
            if success:
                sys.exit(0)
            else:
                sys.exit(1)

# vim: tabstop=4 expandtab shiftwidth=4
